#install.packages("MASS")
#install.packages("lars")
library(MASS)
library(lars)
#diry = "C:/Users/Sangwon/Dropbox/CMU/CURRENT courses/JournalClub/MyNotes/Knockoff/"
#setwd(diry)
source('funs.R')

n = 300
p = 100
k = 30

#############################################
# Rob's example
#############################################
# Settings: N(0,1) noise in design matrix, beta = thirty +-3.5's (signals), and seventy zero (nulls)
#           N(0,1) model noise

set.seed(3)

x = matrix(nrow = n, ncol = p)
for(irow in 1:n){
  for(icol in 1:p){
    x[irow, icol] = rnorm(1,0,1)
  }
}
colsize = sapply(1:p, function(j) sqrt(sum(x[,j]^2)) ,simplify = "array")
x = scale(x, center= FALSE, scale = colsize)

observed_FDR = numeric(100)
observed_power = numeric(100)

###############################################
## Main Program ##
##############################################


#for each run, devise 100
for(b in c(1,5,10,20)){
  cat("b: ",b,fill=TRUE)
  ## Beta is fixed in our single model
  signal_beta = c()
  for(ki in 1:k)   signal_beta = c(signal_beta, (if(runif(1,0,1)>0.5) b else b))
  beta = rep(0,p)
  beta[1:k] = signal_beta #beta[sample(1:1000, 30)] <- signal_beta
  
  
for(runs_i in 1:100){
  cat("run: ", runs_i, "  ")
  noise = rnorm(n=n,mean=0,sd=1)
  y = x%*%beta + noise
## 1. Construct Xtilda
  xt = knockoff(x=x,opt= "equivariance")
## 2. Obtain lasso path
  lasso1 = lars(x=cbind(x,xt), y=y, type = "lar")
  entry = numeric(2*p)
  for(j in 1:length(lasso1$actions))   entry[j] = lasso1$actions[[j]]
  entry = abs(entry)
## 3. Obtain Z and W statistics
  Z = 2*p+1-order(entry)
  W = numeric(p)
  for(j in 1:p)     W[j] = max(Z[j],Z[j+p])*sign(Z[j]-Z[j+p])
#Checking results
  fdr_target = .2
  tlist = unique(abs(W))
#  fdr_est_knockoff = numeric(length(tlist))
  fdr_est_knockoffplus = numeric(length(tlist))

  for(j in 1:length(tlist)){
    tee = tlist[j]
    #fdr_est_knockoff[j] = (sum( W < (-tee))) / max( sum(W >= tee), 1)
    fdr_est_knockoffplus[j] = (1+sum( W < (-tee), na.rm = TRUE)) / max( sum(W >= tee, na.rm = TRUE), 1)
  } 
  Thresh = min(tlist[fdr_est_knockoffplus<fdr_target])

#Checking observed FDR for our dataset
  selected = (1:p)[W > Thresh]
  num_nulls_selected = sum(!( selected %in% (1:k)))
  num_selected = length(selected)
  observed_FDR[runs_i] = num_nulls_selected/num_selected
  
  num_signals_selected = sum( selected %in% (1:k))
  observed_power[runs_i] = num_signals_selected/k

#plotting
pdf(paste0("Knockoffplus_",b,".pdf"), height = 7, width = 14)
  par(mfrow = c(1,2))
  hist(observed_FDR, main = paste0("FDR from simulations \n n=300,p=100 \n k=30,b=",b), col = 'skyblue')
  abline(v = mean(observed_FDR, na.rm=T), col = 'red', lwd = 2)
  legend("topleft", col='red', lwd = 2, legend = round(mean(observed_FDR,na.rm=T),3))
  
  hist(observed_power,main = paste0("Power from simulations \n n=300,p=100 \n k=30,b=",b), col = 'skyblue')
  abline(v = mean(observed_power,na.rm=T), col = 'red', lwd = 2)
  legend("topleft", col='red', lwd = 2, legend = round(mean(observed_power,na.rm=T),3))
dev.off()
}
}


#With the knowledge of true FDR, we see how close/off we are
  Thresh_list = seq(from = 0, to = 5, length=100)
  obs_fdr_list = numeric(100)
  for(m in 1:100){
    obs_fdr_list[m] = sum(!( (1:p)[W > Thresh_list[m]] %in% (1:k)))/k
  }
  plot(x=Thresh_list, y=obs_fdr_list)
  abline(h = .2, col = 'lightgrey')
  abline(v = Thresh, col = 'red')





## todo: Compare 
#1. equivariant construction of Knockoff+ and 
#2. SDP construction of Knockoff+ (may not be easy; see if it works in simpler example)
#3. BHq procedure with (1) original (2) log-factor noise (3) whitened noise